package Parser;

use strict;
use warnings;
use vars qw#%DATAFIELDS#;

use Exporter;
use Parser::WebserverTools qw#%MONTH_NUMBERS#;

@Parser::EXPORT_OK = qw(parse extra %DATAFIELDS);
@Parser::ISA = ('Exporter');

our $Mails = {};
our $Extra =
{
	relay_denied	=> 0,
	tls_hosts	=> {}
};

%DATAFIELDS =
(
	date		=> 'key:date',
	hour		=> 'key:hour',

	sender		=> 'key',
	recipient	=> 'key',

	defer_count	=> 'key:numeric',
	delay		=> 'key:numeric',

	incoming_host	=> 'key:host',
	outgoing_host	=> 'key:host',

	count		=> 'agg',
	bytes		=> 'agg:bytes'
);

# This needs to be done at runtime, since Data uses Setup which relies on
# %DATAFIELDS to be defined  -octo
require Data::Core;
import Data::Core qw#store#;

my $VERSION = '$Id: Postfix.pm,v 1.3 2003/03/11 18:46:46 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub parse
{
	my $line = shift;

	if ($line =~ m#^(\w{3})\s+(\d+) (\d\d):(\d\d):(\d\d) (\S+) postfix/([^\[]+)[^:]+: ([A-F0-9]+): (.+)$#)
	{
		my ($month_name, $day, $hour, $minute, $second,
			$hostname, $service, $id, $line_end) =
		($1, $2, $3, $4, $5, $6, $7, $8, $9);
	
		my $date = sprintf ("%04u-%02u-%02u", (localtime ())[5] + 1900,
			$MONTH_NUMBERS{$month_name}, $day);

		if (!defined ($Mails->{$id}))
		{
			$Mails->{$id} =
			{
				date		=> $date,
				hour		=> $hour,
				sender		=> '*UNKNOWN*',
				recipient	=> '*UNKNOWN*',
				defer_count	=> 0,
				delay		=> 0,
				incoming_host	=> '*UNKNOWN*',
				outgoing_host	=> '*UNKNOWN*',
				count		=> 1,
				bytes		=> 0
			};
		}

		$Mails->{$id}{'date'} = $date;
		$Mails->{$id}{'hour'} = $hour;
		
		if ($line_end =~ m/^to=<([^>]+)>, relay=([^,]+), delay=(\d+), status=(\w+)/)
		{
			my ($to, $relay, $delay, $status) = ($1, $2, $3, $4);

			$Mails->{$id}{'recipient'} = $to;
			if ($Mails->{$id}{'delay'} < $delay)
			{
				$Mails->{$id}{'delay'} = $delay;
			}

			if ($relay =~ m/^([^\[]+)\[([\d\.]+)\]$/)
			{
				my $host = $1;
				my $ip = $2;

				if ($host eq 'unknown')
				{
					$Mails->{$id}{'outgoing_host'} = $ip;
				}
				else
				{
					$Mails->{$id}{'outgoing_host'} = $host;
				}
			}
			elsif ($relay eq 'local')
			{
				$Mails->{$id}{'outgoing_host'} = 'localhost';
			}

			if ($status eq 'sent')
			{
				store_mail ($id);
			}
			elsif ($status eq 'deferred')
			{
				$Mails->{$id}{'defer_count'}++;
			}
			elsif ($status eq 'bounced')
			{
				$Mails->{$id}{'recipient_count'}--;
				if ($Mails->{$id}{'recipient_count'} < 1)
				{
					delete ($Mails->{$id});
				}
			}
			elsif ($::DEBUG)
			{
				print STDERR $/, __FILE__, ": Unknown status: $status";
			}
		}
		elsif ($line_end =~ m/^from=<([^>]*)>, size=(\d+), nrcpt=(\d+)/)
		{
			my ($from, $size, $nrcpt) = ($1, $2, $3, $4);

			$Mails->{$id}{'sender'} = $from if ($from);
			$Mails->{$id}{'bytes'} = $size;
			
			$Mails->{$id}{'recipient_count'} = $nrcpt;
		}
		elsif ($line_end =~ m/client=([^\s,]+)/)
		{
			my $client = $1;

			if ($client =~ m/^([^\[]+)\[([\d\.]+)\]$/)
			{
				my $host = $1;
				my $ip = $2;

				if ($host eq 'unknown')
				{
					$Mails->{$id}{'incoming_host'} = $ip;
				}
				else
				{
					$Mails->{$id}{'incoming_host'} = $host;
				}
			}
			elsif ($::DEBUG)
			{
				print STDERR $/, __FILE__,
				": Unable to parse client string: $client";
			}
		}
	}
	elsif ($line =~ m/Relay access denied/i)
	{
		$Extra->{'relay_denied'}++;
	}
	elsif ($line =~ m/TLS connection established (?:to|from) ([^\[]+)\[([^\]]+)\]/i)
	{
		my $host = $1;
		my $ip = $2;

		my $ident = ($host eq 'unknown' ? $ip : $host);

		$Extra->{'tls_hosts'}{$ident} = 1;
	}
	elsif ($::DEBUG and 0)
	{
		chomp ($line);
		print STDERR $/, __FILE__, ": Unable to parse line: $line";
	}
}

sub store_mail
{
	my $id = shift;
	my $mail = $Mails->{$id};

	store ($mail);
	
	$mail->{'recipient_count'}--;
	if ($mail->{'recipient_count'} < 1)
	{
		delete ($Mails->{$id});
	}
}

sub extra
{
	if ($Extra->{'relay_denied'})
	{
		$::EXTRA->{'Relay access denied'} = sprintf ("%u times", $Extra->{'relay_denied'});
	}

	my $tls_hosts = scalar (keys (%{$Extra->{'tls_hosts'}}));
	if ($tls_hosts)
	{
		$::EXTRA->{'TLS connections'} = sprintf ("%u hosts", $tls_hosts);
	}
}
